/*******************************************************************************
* FILE NAME: auto_routines.c
*
* DESCRIPTION:
*   Master control of the autonomous scripts. This is a framework for function-driven autonomous.
*
*   Functions are not aware of what has gone before or what will come after. They are
*     completely independent of other actions or combinations of actions of the scripts.
*   Functions can be 1-cycle (immediate) or take several cycles to complete
*   Functions only set the environment once-per-cycle, they never usurp Master loop control.
*   Functions can be overridden by Master control
*   
*   Each function:
*     1) Determines it's own success, failure, or incomplete status
*     2) Has an "Init" condition where initial parameters are set and remembered.
*          Original command arguments are preserved within the functions themselves
*          and are NOT supplied on subsequent calls.
*     3) Has a "Completed" condition where control is passed to the n
*     4) May have a "Continue" condition where it has not yet completed it's task
*     5) Is responsible for deciding when it has completed it's given task
*     6) Maintains any intermediate values required to be carried over from cycle to cycle.
*        
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "stdio.h"

#define VARIABLE_DECLARE  /* causes certain global variables to be declared only once */
  #include "auto_routines.h"
#undef VARIABLE_DECLARE

#include "targeting.h"
#include "user_routines.h"
#include "clock_routines.h"
#include "utilities.h"
#include "auto_scripts.h"  // All autonomous scripts reside here

/* Status is set by the individual auto functions */

/*******************************************************************************
* FUNCTION NAME: User_Autonomous_Code
* PURPOSE:       This is called constantly and checks to see if it is time to
*				  execute a function.
*******************************************************************************/
void User_Autonomous_Code(void)
{
  static unsigned int swtc;  //set equal to the switch input

  /** Autonomous is selected by the auto mode selector **/
  /* Only check this switch once at the start */
  if (AutoFirstTime)
  { /* Initialize the first time autonomous starts */
    AutoFirstTime = FALSE;
    // What autonomous script are we running? (10 selectable)
    swtc = AUTOMODE;  // Auto select switch 0-9
printf("swtc=%d\r", swtc);
    holdflag = FALSE;

    Drive_Status=COMPLETE; Shooter_Status=COMPLETE; Joint_Status=COMPLETE;
    lastDriveCmd=ENDCMD;   lastShooterCmd=ENDCMD;   lastJointCmd=ENDCMD;
    DriveCmd=0;            ShooterCmd=0;            JointCmd=0; // Current cmd # in script
    cmdNum=0; lastcmdNum=99;
    
    PAN_MOTOR  = NEUTRAL;
    TILT_MOTOR = NEUTRAL;
    WHL_R1 = WHL_R2 = NEUTRAL;
    WHL_L1 = WHL_L2 = NEUTRAL;

  } /* End FirstTime */

  // What is the robot starting position?

  if (rc_dig_in11 == 0)
  {
    start_position = STARTLEFT;
//    printf(" ***left*** ");
  }
  else if (rc_dig_in12 == 0)
  {
    start_position = STARTCENTER;
//    printf(" ***center*** ");
  }
  else if (rc_dig_in13 == 0)
  {
    start_position = STARTRIGHT;
//    printf(" ***right*** ");
  }
  else
  {
    start_position = STARTCENTER;  // default
//    printf(" ***CENTER (default)*** ");
  }

//***** DEBUG *****
//  swtc = 9;
//  start_position = STARTLEFT;
//***** END DEBUG *******

    /* Copy selected script into workspace */
    switch(swtc)
	  {
   	    case 0: 
              if (start_position == STARTRIGHT)
              {
                Switch_Auto(AUTO_RIGHT_0);
              }
              else if (start_position == STARTCENTER)
              {
                Switch_Auto(AUTO_CENTER_0);
              }
              else
              {
                Switch_Auto(AUTO_LEFT_0);
              }
			  break;
  		case 1: 
	    default:
              if (start_position == STARTRIGHT)
                Switch_Auto(AUTO_RIGHT_1);
              else if (start_position == STARTCENTER)
                Switch_Auto(AUTO_CENTER_1);
              else
                Switch_Auto(AUTO_LEFT_1);
		  	break;
  		case 2: 
              if (start_position == STARTRIGHT)
                Switch_Auto(AUTO_RIGHT_2);
              else if (start_position == STARTCENTER)
                Switch_Auto(AUTO_CENTER_2);
              else
                Switch_Auto(AUTO_LEFT_2);
		  	break;
  		case 3: 
              if (start_position == STARTRIGHT)
                Switch_Auto(AUTO_RIGHT_3);
              else if (start_position == STARTCENTER)
                Switch_Auto(AUTO_CENTER_3);
              else
                Switch_Auto(AUTO_LEFT_3);
		  	break;
  		case 4: 
              if (start_position == STARTRIGHT)
                Switch_Auto(AUTO_RIGHT_4);
              else if (start_position == STARTCENTER)
                Switch_Auto(AUTO_CENTER_4);
              else
                Switch_Auto(AUTO_LEFT_4);
  	  	    break;
  		case 5: 
              if (start_position == STARTRIGHT)
                Switch_Auto(AUTO_RIGHT_5);
              else if (start_position == STARTCENTER)
                Switch_Auto(AUTO_CENTER_5);
              else
                Switch_Auto(AUTO_LEFT_5);
		  	break;
  		case 6: 
              if (start_position == STARTRIGHT)
                Switch_Auto(AUTO_RIGHT_6);
              else if (start_position == STARTCENTER)
                Switch_Auto(AUTO_CENTER_6);
              else
                Switch_Auto(AUTO_LEFT_6);
		  	break;
  		case 7: 
              if (start_position == STARTRIGHT)
                Switch_Auto(AUTO_RIGHT_7);
              else if (start_position == STARTCENTER)
                Switch_Auto(AUTO_CENTER_7);
              else
                Switch_Auto(AUTO_LEFT_7);
		  	break;
  		case 8: 
              if (start_position == STARTRIGHT)
                Switch_Auto(AUTO_RIGHT_8);
              else if (start_position == STARTCENTER)
                Switch_Auto(AUTO_CENTER_8);
              else
                Switch_Auto(AUTO_LEFT_8);
		  	break;
  		case 9: 
              if (start_position == STARTRIGHT)
                Switch_Auto(AUTO_RIGHT_9);
              else if (start_position == STARTCENTER)
                Switch_Auto(AUTO_CENTER_9);
              else
                Switch_Auto(AUTO_LEFT_9);
		  	break;
       }
  /* 4 motor drive */
  WHL_L2 = WHL_L1;	//set the second motor to same
  WHL_R2 = WHL_R1 *= MOTORSIGN;  //same note as above...
}


/*******************************************************************************
* FUNCTION NAME: Switch_Auto
* PURPOSE:       Process an autonomous command script
* Original command arguments are preserved within the functions themselves
*******************************************************************************/
void Switch_Auto(rom const int *CommandScript)
{
  int speed, distance, time, direction, heading, distance_right, distance_left, faster;
  int setting, nextCmd, unused;
  int i, j, k;

  /****************************************/
  /* Revisit commands already in progress */
  /****************************************/

  /*******************************/
  /************ Drive ************/
  switch(lastDriveCmd)
  {
      default:  // includes JOINT cmds
      case ENDCMD:  // Do nothing - no drive cmds yet
//          Auto_Hold_Position();  // Reversed logic? Causes runaway
          break;    
      case DFWD:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Forward_Time(time, speed);
          break;
      case DBCK:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Backward_Time(time, speed);
          break;
        case DSPIN:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          direction = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIRECTION];
          Drive_Spin_In_Place_Time(time, speed, direction);
          break;
      case DSTOP:
          Drive_Stop();
          break;
      case GTRN:
          heading = CommandScript[DriveCmd*AUTO_WIDTH + CMDGHEADING];
          Gyro_Turn(heading);
          break;
      case GTRNS:
          heading = CommandScript[DriveCmd*AUTO_WIDTH + CMDGHEADING];
          Gyro_Turn_Sloppy(heading);
          break;
      case DISTFWD:
          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Forward_Dist(distance, speed);
          break;
  }
  /*********************************/
  /************** Shooter **********/

  switch(lastShooterCmd)
  {
     default:
     case ENDCMD:  // Do nothing - no shooter cmds yet
          break;    
      case SHOOTERON:
          Shooter_On();
          break;
      case SHOOTEROFF:
          Shooter_Off();
          break;
      case FIRE:
          Fire_Poof();
          break;
      case SETPAN:
          setting = CommandScript[ShooterCmd*AUTO_WIDTH + CMDSETTING];
          Set_Pan_Position(setting);
          break;
      case PANTRACK:
          Pan_Track();
          break;
      case TLOCK:
          Target_Lock();
          break;
      case CONVEYON:
          Conveyor_On();
          break;
      case CONVEYOFF:
          Conveyor_Off();
          break;
  }

  /*******************************/
  /************* Joint ***********/
  switch(lastJointCmd)
  {
      default:
      case ENDCMD:  // Do nothing - no joint cmds yet
          break;    
      case DELAY:
          time = CommandScript[JointCmd*AUTO_WIDTH + CMDTIME];
          Delay(time);
          break;
      case SYNC:
          time = CommandScript[JointCmd*AUTO_WIDTH + CMDTIME];
          Sync_Drive_Shooter(time);
          break;
  }

/******************************/
/** Process any new commands **/
/******************************/

  /* Look at next command on each queue */
  nextCmd = CommandScript[cmdNum*AUTO_WIDTH + CMD];

  if (nextCmd != ENDCMD)
  {
   if (nextCmd <= GTRNS) // Simply the last drive command
   {
     if (Drive_Status == COMPLETE)
    {    /* Start next Drive cmd */
      DriveCmd = cmdNum;
      cmdNum++;  // Get ready for next command
      holdflag = FALSE;
      switch(nextCmd)
      { 
        default:
        case ENDCMD:  // Do nothing - no drive cmds yet
          break;    
        case DFWD:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Forward_Time(time, speed);
          break;
        case DBCK:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Backward_Time(time, speed);
          break;
        case DSPIN:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          direction = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIRECTION];
          Drive_Spin_In_Place_Time(time, speed, direction);
          break;
        case DSTOP:
          Drive_Stop();
          break;
        case GTRN:
          heading = CommandScript[DriveCmd*AUTO_WIDTH + CMDGHEADING];
          Gyro_Turn(heading);
          break;
      case GTRNS:
          heading = CommandScript[DriveCmd*AUTO_WIDTH + CMDGHEADING];
          Gyro_Turn_Sloppy(heading);
          break;
      case DISTFWD:
          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Forward_Dist(distance, speed);
      }
     }
    }
    else if (nextCmd >= SHOOTEROFF)  // Simply the first shooter command
    {
     if (Shooter_Status == COMPLETE)
     {    /* Start next Shooter cmd */
      ShooterCmd = cmdNum;
      cmdNum++;  // Get ready for next command
printf(" nextCmd=%d, ",nextCmd);
      switch(nextCmd)
      {
        default:
        case ENDCMD:  // Do nothing - no shooter cmds yet
          break;    
      case SHOOTERON:
          Shooter_On();
          break;
      case SHOOTEROFF:
          Shooter_Off();
          break;
      case FIRE:
          Fire_Poof();
          break;
      case SETPAN:
          setting = CommandScript[ShooterCmd*AUTO_WIDTH + CMDSETTING];
          Set_Pan_Position(setting);
          break;
      case PANTRACK:
          Pan_Track();
          break;
      case TLOCK:
          Target_Lock();
          break;
      case CONVEYON:
          Conveyor_On();
          break;
      case CONVEYOFF:
          Conveyor_Off();
          break;
      }
     }
    }
    else if (Joint_Status == COMPLETE)
    {    /* Start next Joint cmd */
      JointCmd = cmdNum;
      cmdNum++;  // Get ready for next command
      switch(nextCmd)
      {
        default:
        case ENDCMD:  // Do nothing - no joint cmds yet
          break;    
        case DELAY:
          time = CommandScript[JointCmd*AUTO_WIDTH + CMDTIME];
          Delay(time);
          break;
        case SYNC:
          time = CommandScript[JointCmd*AUTO_WIDTH + CMDTIME];
          Sync_Drive_Shooter(time);
          break;
      }
    }
  } /* End ENDCMD check */
}
