/*******************************************************************************
* FILE NAME: auto_shooter_routines.c
*
* DESCRIPTION:
*   Autonomous shooter and conveyor functions
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "stdio.h"

#include "auto_routines.h"
#include "user_routines.h"
#include "clock_routines.h"
#include "gyro.h"
#include "utilities.h"
#include "velocity_control.h"
#include "targeting.h"


/*******************************************************************************
* FUNCTION NAME: Shooter_On
* PURPOSE:       Turn the shooter on, power-up the wheels
*******************************************************************************/
void Shooter_On()
{
  ShooterOn=TRUE;
}


/*******************************************************************************
* FUNCTION NAME: Shooter_Off
* PURPOSE:       Turn the shooter off
*******************************************************************************/
void Shooter_Off()
{
  ShooterOn=FALSE;
  UpToSpeed = FALSE;
}


/*******************************************************************************
* FUNCTION NAME: Set_Pan_Position
* PURPOSE:       Move pan to a specified position.
*     pos - the pan position as would normally be sent from the OI position control
*           OIPOTMIN is all the way left, OIPOTMAX is all the way right
*******************************************************************************/
void Set_Pan_Position(int pos)
{
   static int position;  //Target position of the pot on the pan
   int pan_motor;

   //*---------- Shooter Pan gimbal --------------------------------------------
   //*--------------------------------------------------------------------------
  if (lastShooterCmd != SETPAN)
  {
    printf("SETPAN: 1\r");
    lastShooterCmd = SETPAN;
    // Base on OI dial logic and map to pan potentiometer
    position = (long)(pos - OIPOTMIN) * (PANPOTMAX - PANPOTMIN) / (OIPOTMAX - OIPOTMIN) + PANPOTMIN;  
  }

  Pan_By_Pot(position, &pan_motor);
  PAN_MOTOR = pan_motor;

  if (PAN_MOTOR != 0)
  { // Keep going
    Shooter_Status = INCOMPLETE;
  }
  else
  { // All done (move on to next shooter command)
    printf(" SETPAN: Completed \r");
    Shooter_Status = COMPLETE;
    lastShooterCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    PAN_MOTOR = NEUTRAL;
  }
  printf("SETPAN: PAN_MOTOR=%d, PANPOT=%d, position=%d\r", PAN_MOTOR, PANPOT, position);
}


/*******************************************************************************
* FUNCTION NAME: Fire_Poof
* PURPOSE:       Fire poof ball
*******************************************************************************/
#define BALLLAUNCHDELAY 1  // 1/2 second

void Fire_Poof()
{
  static long Time_ball_launch=0;

  if (lastShooterCmd != FIRE)
  {
    printf("FIRE: - ");
    lastShooterCmd = FIRE;
    Shooter_Status = INCOMPLETE;
    Time_ball_launch=0;
  }

  if (BALLLAUNCH==0) // No launch yet
  {
    if(BALLREADY && UpToSpeed) // Ready to fire
    {
      BALLLAUNCH = 1; // Fire
      Time_ball_launch = 1;
//      Time_ball_launch = Clockds + BALLLAUNCHDELAY;
    }
  }
//  else if ((!PISTONUP) || (Time_ball_launch < Clockds))
//  else 
//  {
//    ++Time_ball_launch;
  //  if (Time_ball_launch > 10)
  else if ((!PISTONUP))
  {
    printf("PISTONUP=%d, time=%d, clock=%d\r", (int)PISTONUP, (int)Time_ball_launch, (int)Clockds);
    BALLLAUNCH = 0;
    // All done (move on to next shooter command)
    Shooter_Status = COMPLETE;
    lastShooterCmd = ENDCMD;  // In case there are two of the same type cmd in a row
  }
//  }
}


/*******************************************************************************
* FUNCTION NAME: Pan_Track
* PURPOSE:       Track the target light with the camera
*******************************************************************************/
void Pan_Track()
{
  char completed;
  int pan_motor;

  if (lastShooterCmd != PANTRACK)
  {
//    printf("PANTRACK: 1\r");
    lastShooterCmd = PANTRACK;
  }

  completed = GimbalTrackPan(&pan_motor);
  PAN_MOTOR = pan_motor;

  if (completed)
  {
 // All done (move on to next shooter command)
    Shooter_Status = COMPLETE;
    lastShooterCmd = ENDCMD;  // In case there are two of the same type cmd in a row
  }
  else
    Shooter_Status = INCOMPLETE;

}

/*******************************************************************************
* FUNCTION NAME: Target_Lock
* PURPOSE:       Are we locked onto the target?
*******************************************************************************/
void Target_Lock()
{
  char completed;
  int pan_motor;

  if (lastShooterCmd != TLOCK)
  {
//    printf("TLOCK: 1\r");
    lastShooterCmd = TLOCK;
  }

  completed = GimbalTrackPan(&pan_motor);

  if (completed)
  {
 // All done (move on to next shooter command)
    Shooter_Status = COMPLETE;
    lastShooterCmd = ENDCMD;  // In case there are two of the same type cmd in a row
  }
  else
    Shooter_Status = INCOMPLETE;

}


/*******************************************************************************
* FUNCTION NAME: Conveyor_On
* PURPOSE:       Turn the conveyor on
*******************************************************************************/
void Conveyor_On()
{
  HARVESTER_MOTOR = -127;
  printf("***** Conveyor on\r");
}


/*******************************************************************************
* FUNCTION NAME: Conveyor_Off
* PURPOSE:       Turn the conveyor off
*******************************************************************************/
void Conveyor_Off()
{
  HARVESTER_MOTOR = 0;
  printf("***** Conveyor off\r");
}
