/*******************************************************************************
* FILE NAME: velocity_control.c
*
* DESCRIPTION:
*     This controls the shooter wheels to startup and maintain speed.
*******************************************************************************/

#define VARIABLE_DECLARE  /* causes certain global variables to be declared only once */
  #include "velocity_control.h"
#undef VARIABLE_DECLARE

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines.h"
#include "utilities.h"
#include <stdio.h>

#include "clock_routines.h"

/*******************************************************************************
* FUNCTION NAME: VelocityControl
* PURPOSE:  Maintain a constant speed of the shooter wheels
*           The challenge is regaining our target speed eveytime the balls 
*           momentarily slow down the motor. However, this will also act to dampen
*           the normal motor/tachometer fluctuations
*           The shooter is unique in that the wheels will attempt to free-spin at a
*           higher than desired speed, and it will be hard to slow them down.
* NOTES:
*      May need to deadband around the target speed to make the derivative more effective
*      at a rapid/drastic response when a ball engages and slows the motor.
*      500Hz typical sensor return at speed
*  2800-2900 rpm is our design target
* volts     RPM     analog  ~pwm
*   0.0        0      0      127
*   0.98    1000    200      
*   1.45    1500    297      
*   1.95    2000    399      
*   2.5     2500    512      190
*   3.0     3000    614      
*   3.52    3500    720      
*   4.1     4000    839      
*   5.0     4878    1023     254       -in theory
*******************************************************************************/

#define KP 35/10  // 23 works well
#define KI 1      // 1 is okay
#define KD 5      // 5 works well
#define MAXERROR 130 //(130 works@KI=1) Integral - anything * KI > 127 is wasted. Set it high for tinkering w/KI
#define dTHRESHOLD     4
#define dTHRESHOLDHIGH 8

#define MAXRPM 4878       // Arrived at by measurement
#define SHOOTERNEUTRAL  127 // In case neutral isn't neutral
#define SENSOR_OFFSET 0  // (48) Tach reads this value when at rest
#define RAMPUP 4   // Was 2 - Start the motor up slowly rather than immediate full power

#define TARGETSPEED 190  // (189) 185 - 11.73 m/sec 195 =13.25 m/sec

void VelocityControl(void)
{
  unsigned char range;
  unsigned int target_speed=0, current_speed=0;
  int error, derivative, proportional, integral;
  long temp, target_rpm, current_rpm;
  static long last_speed=0; // The previous speed used for Derivative
  static int Cumerror=0;    // The accumulated error used for Integral
  static char overspeed=0;  // Flag when motor passes target speed - going up or going down
  static char startup=0;    // Signals a slow start for the motor
  int tempMotor;

//  static long timecount=0; //Debug to slow down printfs

  // Collect our inputs from the user and sensor
//  target_speed  = (unsigned int)Get_Analog_Value(rc_ana_in07);  //Used as the target value to hit. This is our PID offset.
  target_speed  = TARGETSPEED; //Used as the target value to hit. This is our PID offset.
  current_speed = (unsigned int)SHOOTERTACH - SENSOR_OFFSET;  //Tachometer motor speed (may need filtering)
  current_speed = current_speed & ~3;  // smooth the tach input by zeroing the last two bits (deadband of 3) 
//printf("current_speed1=%d, ", current_speed); // tach analog range is ~47 to 1023

//***************** DEBUG ***********************
// Use 4 pots to set target_speed, KP, KI, KD
//  KP = (long) 50 * Get_Analog_Value(rc_ana_in03)/1023;  //Value is in tenths 0-50

//  KI = (long) 50 * Get_Analog_Value(rc_ana_in04)/1023;  //Value is 0-50 in tenths
//  KD = (long) 10 * Get_Analog_Value(rc_ana_in05)/1023;  //Value is 0-10      
//  KP = 23;
//  KI = 1;
//  KD = 5;
//***************** DEBUG ***********************

  // Calculated just for "idle" interest
//  target_rpm  = (long) target_speed  * MAXRPM / 1023; 
//  current_rpm = (long) current_speed * MAXRPM / (1023-SENSOR_OFFSET);

  range = 254 - SHOOTERNEUTRAL;
//  target_speed  = (unsigned int)(range * (long)target_speed/1023) + SHOOTERNEUTRAL;  //Used as the target value to hit. This is our PID offset.
  current_speed = (unsigned int)(range * (long)current_speed/(1023-SENSOR_OFFSET)) + SHOOTERNEUTRAL; //Tachometer - current motor speed (may need filtering)
//printf("current_speed2=%d, ", current_speed); // tach analog range is ~47 to 1023

  //----------------------------------------
  //********* Proportional correction *********
  // p will handle the "too fast" condition alone (no i or d)
  error = (int) target_speed - current_speed; // positive is too slow, negative is too fast 
  proportional = (long) error * KP;


  //********* Integrate error *********
  //-------------
// Must preserve the integral if we're low on battery and must maintain our speed
#ifdef MLM  //Comment out the following code
  // Restart (zero) our cumulative error whenever we cross the target speed threshold
  if (current_speed == target_speed)
  {
    Cumerror = 0;
//    UpToSpeed = TRUE;
  }
  else if ((current_speed > target_speed) && (overspeed==0))
  {
    Cumerror = 0;
    overspeed = 1;
//    UpToSpeed = TRUE;
  }
  else if ((current_speed < target_speed) && (overspeed==1))
  {
    Cumerror = 0;
    overspeed = 0;
//    UpToSpeed = TRUE;
  }
#endif //MLM
  //-------------
  integral = Cumerror * (int) KI;  // positive is too slow, negative is too fast
  Cumerror += error;
  if (Cumerror >  MAXERROR)  Cumerror =  MAXERROR;
  if (Cumerror < 0) Cumerror = 0; //Ignore negative error - let the wheels coast

  //********* Derivative dampening *********
  // Sudden reaction to sudden changes, i.e., it will spike when a ball engages
  if (error == 0)
  {
    derivative = 0;
    last_speed = current_speed;
  }
  else
  {
    temp = last_speed - current_speed;
    derivative = temp * (int)KD; // positive is too slow, negative is too fast 
    if (derivative < 0) derivative = 0; //Chop off any negative response; Balls require only positive correction
    if (temp < dTHRESHOLD) derivative = 0; //Deadband around the target set point
    if (temp > dTHRESHOLDHIGH)  // On spikes simply force the Integral to follow the Derivative high
    {
      Cumerror += derivative; // This will keep full power on the motor until the sensor gets above the target speed
      if (Cumerror >  MAXERROR)  Cumerror =  MAXERROR;
//      UpToSpeed = FALSE;
    }
    last_speed = current_speed;
  }

  //********* It all comes together ********
  temp = (long) target_speed + (proportional + integral + derivative); //Maintains the target offset 
      // It's important to note that target_speed looks like a pwm value, but is really a sensor value.
  if (temp > 254) temp = 254;  // Don't let it be bigger than possible
  if (temp < SHOOTERNEUTRAL) temp = SHOOTERNEUTRAL; // And never run the motor backwards

  //*********************************************************************
  // Don't allow jackrabbit starts
  // Slow shooter startup to avoid unnecessary stress to the transmission
  // *** Override of PID values will normally take less than 2.5 second
  if (startup == 0)
  {
    if (temp < target_speed)
    {
      temp += RAMPUP; 
    }
    else
    {
      startup = 1;
    }
  }
  SHOOTER_MOTOR = temp - 127;  // convert to signed math
  if (current_speed > target_speed)
    UpToSpeed = TRUE;
  else
    UpToSpeed = FALSE;

  //------------- DEBUG ---------------------------
//  if (timecount%1 == 0) // Slow down the prints if desired
  {
//printf("current_speed3=%d, ", current_speed); // tach analog range is ~47 to 1023
//    printf("KP,%d/10, KI,%d, KD,%d, ", 10*KP, KI, KD);
//    printf("Crpm,%d, ", (int) current_rpm);
    //printf("TS,%d,CS,%d, ", (int)target_speed, (int)current_speed);
    //printf("pwm,%d, ", SHOOTER_MOTOR);
    //printf("p,%d, i,%d, d,%d, ", proportional, integral, derivative);
    //printf("\r");
  }
//  timecount++;
}


/*******************************************************************************
* FUNCTION NAME: DrumsAtSpeed
* PURPOSE:  Check to see if the shooter is ready to shoot.
*******************************************************************************/
char DrumsAtSpeed()
{
   return(UpToSpeed);
}
